from enum import Enum


class FilingType(Enum):
    """Available filing types to be used when creating Filing object.

    .. versionadded:: 0.1.5

    """
    FILING_1 = '1'
    FILING_10 = '10'
    FILING_1012B = '10-12B'
    FILING_1012B_AMEND = '10-12B/A'
    FILING_1012G = '10-12G'
    FILING_1012G_AMEND = '10-12G/A'
    FILING_10C = '10-c'
    FILING_10D = '10-D'
    FILING_10D_AMEND = '10-D/A'
    FILING_10K = '10-K'
    FILING_10K405 = '10-k405'
    FILING_10KSB = '10ksb'
    FILING_10KSB40 = '10ksb40'
    FILING_10KSB405 = '10ksb405'
    FILING_10KT = '10-KT'
    FILING_10KT405 = '10kt405'
    FILING_10KT_AMEND = '10-KT/A'
    FILING_10K_AMEND = '10-K/A'
    FILING_10M = '10-m'
    FILING_10Q = '10-Q'
    FILING_10QSB = '10qsb'
    FILING_10QT = '10-QT'
    FILING_10QT_AMEND = '10-QT/A'
    FILING_10Q_AMEND = '10-Q/A'
    FILING_10SB12B = '10sb12b'
    FILING_10SB12G = '10sb12g'
    FILING_11K = '11-K'
    FILING_11KT = '11-KT'
    FILING_11KT_AMEND = '11-KT/A'
    FILING_11K_AMEND = '11-K/A'
    FILING_12B25 = '12b-25'
    FILING_12G2 = '12g-2'
    FILING_12G32A = '12g3-2a'
    FILING_12G32B = '12g3-2b'
    FILING_12G32BR = '12g32br'
    FILING_13F = '13f'
    FILING_13FCONP = '13fconp'
    FILING_13FE = '13f-e'
    FILING_13FHR = '13F-HR'
    FILING_13FHR_AMEND = '13F-HR/A'
    FILING_13FNT = '13F-NT'
    FILING_13FNT_AMEND = '13F-NT/A'
    FILING_13H = '13H'
    FILING_13HA = '13H-A'
    FILING_13HI = '13H-I'
    FILING_13HQ = '13H-Q'
    FILING_13HR = '13H-R'
    FILING_13HT = '13H-T'
    FILING_144 = '144'
    FILING_144_AMEND = '144/A'
    FILING_15 = '15'
    FILING_1512B = '15-12B'
    FILING_1512B_AMEND = '15-12B/A'
    FILING_1512G = '15-12G'
    FILING_1512G_AMEND = '15-12G/A'
    FILING_1515D = '15-15D'
    FILING_1515D_AMEND = '15-15D/A'
    FILING_15F = '15f'
    FILING_15F12B = '15F-12B'
    FILING_15F12B_AMEND = '15F-12B/A'
    FILING_15F12G = '15F-12G'
    FILING_15F12G_AMEND = '15F-12G/A'
    FILING_15F15D = '15F-15D'
    FILING_15F15D_AMEND = '15F-15D/A'
    FILING_17H = '17-h'
    FILING_17HACON = '17HACON'
    FILING_17HACON_AMEND = '17HACON/A'
    FILING_17HQCON = '17HQCON'
    FILING_17HQCON_AMEND = '17HQCON/A'
    FILING_18 = '18'
    FILING_1812B = '18-12B'
    FILING_1812B_AMEND = '18-12B/A'
    FILING_1812G = '18-12G'
    FILING_1812G_AMEND = '18-12G/A'
    FILING_18K = '18-K'
    FILING_18K_AMEND = '18-K/A'
    FILING_19B = '19-b'
    FILING_19B4 = '19b-4'
    FILING_19B4E = '19b-4(e)'
    FILING_19B7 = '19b-7'
    FILING_1A = '1-A'
    FILING_1APOS = '1-A POS'
    FILING_1AW = '1-A-W'
    FILING_1AW_AMEND = '1-A-W/A'
    FILING_1A_AMEND = '1-A/A'
    FILING_1E = '1-E'
    FILING_1EAD = '1-E AD'
    FILING_1EAD_AMEND = '1-E AD/A'
    FILING_1E_AMEND = '1-E/A'
    FILING_1K = '1-K'
    FILING_1K_AMEND = '1-K/A'
    FILING_1N = '1-n'
    FILING_1SA = '1-SA'
    FILING_1SA_AMEND = '1-SA/A'
    FILING_1U = '1-U'
    FILING_1U_AMEND = '1-U/A'
    FILING_1Z = '1-Z'
    FILING_1ZW = '1-Z-W'
    FILING_1ZW_AMEND = '1-Z-W/A'
    FILING_1Z_AMEND = '1-Z/A'
    FILING_20F = '20-F'
    FILING_20FR = '20-fr'
    FILING_20FR12B = '20FR12B'
    FILING_20FR12B_AMEND = '20FR12B/A'
    FILING_20FR12G = '20FR12G'
    FILING_20FR12G_AMEND = '20FR12G/A'
    FILING_20F_AMEND = '20-F/A'
    FILING_24F1 = '24f-1'
    FILING_24F2 = '24f-2'
    FILING_24F2EL = '24f-2el'
    FILING_24F2NT = '24F-2NT'
    FILING_24F2NT_AMEND = '24F-2NT/A'
    FILING_24F2TM = '24f-2tm'
    FILING_25 = '25'
    FILING_253G1 = '253G1'
    FILING_253G2 = '253G2'
    FILING_253G3 = '253G3'
    FILING_253G4 = '253G4'
    FILING_25NSE = '25-NSE'
    FILING_25NSE_AMEND = '25-NSE/A'
    FILING_25_AMEND = '25/A'
    FILING_26 = '26'
    FILING_27 = '27'
    FILING_28 = '28'
    FILING_2A = '2-a'
    FILING_2AF = '2-af'
    FILING_2E = '2-E'
    FILING_2E_AMEND = '2-E/A'
    FILING_3 = '3'
    FILING_305B2 = '305B2'
    FILING_305B2_AMEND = '305B2/A'
    FILING_3412H = '34-12h'
    FILING_3436CF = '34-36cf'
    FILING_3436MR = '34-36mr'
    FILING_3511 = '35-11'
    FILING_352 = '35-2'
    FILING_353 = '35-3'
    FILING_357B = '35-7b'
    FILING_35APP = '35-app'
    FILING_35CERT = '35-cert'
    FILING_3910B2 = '39-10b2'
    FILING_39304C = '39-304c'
    FILING_39304D = '39-304d'
    FILING_39310B = '39-310b'
    FILING_3_AMEND = '3/A'
    FILING_4 = '4'
    FILING_4017F1 = '40-17F1'
    FILING_4017F1_AMEND = '40-17F1/A'
    FILING_4017F2 = '40-17F2'
    FILING_4017F2_AMEND = '40-17F2/A'
    FILING_4017G = '40-17G'
    FILING_4017GCS = '40-17GCS'
    FILING_4017GCS_AMEND = '4017GCS/A'
    FILING_4017G_AMEND = '40-17G/A'
    FILING_40202A = '40-202a'
    FILING_40203A = '40-203a'
    FILING_40205A = '40-205a'
    FILING_40205E = '40-205e'
    FILING_40206A = '40-206a'
    FILING_4024B2 = '40-24B2'
    FILING_4024B2_AMEND = '40-24B2/A'
    FILING_4033 = '40-33'
    FILING_4033_AMEND = '40-33/A'
    FILING_406B = '40-6B'
    FILING_406B_AMEND = '40-6B/A'
    FILING_406C = '40-6c'
    FILING_408B25 = '40-8B25'
    FILING_408F2 = '40-8F-2'
    FILING_408F2_AMEND = '40-8F-2/A'
    FILING_408FA = '40-8f-a'
    FILING_408FB = '40-8f-b'
    FILING_408FC = '40-8fc'
    FILING_408FL = '40-8f-l'
    FILING_408FM = '40-8f-m'
    FILING_40APP = '40-APP'
    FILING_40APP_AMEND = '40-APP/A'
    FILING_40F = '40-F'
    FILING_40FR12B = '40FR12B'
    FILING_40FR12B_AMEND = '40FR12B/A'
    FILING_40FR12G = '40FR12G'
    FILING_40FR12G_AMEND = '40FR12G/A'
    FILING_40F_AMEND = '40-F/A'
    FILING_40OIP = '40-OIP'
    FILING_40OIP_AMEND = '40-OIP/A'
    FILING_40RPT = '40-rpt'
    FILING_424A = '424A'
    FILING_424B1 = '424B1'
    FILING_424B2 = '424B2'
    FILING_424B3 = '424B3'
    FILING_424B4 = '424B4'
    FILING_424B5 = '424B5'
    FILING_424B7 = '424B7'
    FILING_424B8 = '424B8'
    FILING_424H = '424H'
    FILING_424H_AMEND = '424H/A'
    FILING_425 = '425'
    FILING_45B3 = '45b-3'
    FILING_485A24E = '485a24e'
    FILING_485A24F = '485a24f'
    FILING_485APOS = '485APOS'
    FILING_485B24E = '485b24e'
    FILING_485B24F = '485b24f'
    FILING_485BPOS = '485BPOS'
    FILING_485BXT = '485BXT'
    FILING_485BXTF = '485bxtf'
    FILING_486A24E = '486a24e'
    FILING_486APOS = '486APOS'
    FILING_486B24E = '486b24e'
    FILING_486BPOS = '486BPOS'
    FILING_486BXT = '486BXT'
    FILING_487 = '487'
    FILING_497 = '497'
    FILING_497AD = '497AD'
    FILING_497H2 = '497H2'
    FILING_497J = '497J'
    FILING_497K = '497K'
    FILING_497K1 = '497k1'
    FILING_497K2 = '497k2'
    FILING_497K3A = '497k3a'
    FILING_497K3B = '497k3b'
    FILING_497VPI = '497VPI'
    FILING_497VPU = '497VPU'
    FILING_4_AMEND = '4/A'
    FILING_5 = '5'
    FILING_5_AMEND = '5/A'
    FILING_6B_NTC = '6b ntc'
    FILING_6B_ORDR = '6b ordr'
    FILING_6K = '6-K'
    FILING_6K_AMEND = '6-K/A'
    FILING_7A = '7-a'
    FILING_8A12B = '8-A12B'
    FILING_8A12BEF = '8a12bef'
    FILING_8A12BT = '8a12bt'
    FILING_8A12B_AMEND = '8-A12B/A'
    FILING_8A12G = '8-A12G'
    FILING_8A12G_AMEND = '8-A12G/A'
    FILING_8B12B = '8-b12b'
    FILING_8B12G = '8-b12g'
    FILING_8F2_NTC = '8f-2 ntc'
    FILING_8F2_ORDR = '8f-2 ordr'
    FILING_8K = '8-K'
    FILING_8K12B = '8-K12B'
    FILING_8K12B_AMEND = '8-K12B/A'
    FILING_8K12G3 = '8-K12G3'
    FILING_8K12G3_AMEND = '8-K12G3/A'
    FILING_8K15D5 = '8-K15D5'
    FILING_8K15D5_AMEND = '8-K15D5/A'
    FILING_8K_AMEND = '8-K/A'
    FILING_8M = '8-m'
    FILING_9M = '9-m'
    FILING_ABS15G = 'ABS-15G'
    FILING_ABS15G_AMEND = 'ABS-15G/A'
    FILING_ABSEE = 'ABS-EE'
    FILING_ABSEE_AMEND = 'ABS-EE/A'
    FILING_ADB = 'adb'
    FILING_ADNMTL = 'adn-mtl'
    FILING_ADV = 'adv'
    FILING_ADVCO = 'advco'
    FILING_ADVE = 'adv-e'
    FILING_ADVH = 'adv-h'
    FILING_ADVHC = 'adv-h-c'
    FILING_ADVHT = 'adv-h-t'
    FILING_ADVNR = 'adv-nr'
    FILING_ADVW = 'adv-w'
    FILING_AFDB = 'afdb'
    FILING_ANNLRPT = 'ANNLRPT'
    FILING_ANNLRPT_AMEND = 'ANNLRPT/A'
    FILING_APPWD = 'APP WD'
    FILING_APPWDAPPWD_AMEND = 'APP WD APP WD/A'
    FILING_APPWD_AMEND = 'APP WD/A'
    FILING_APP_NTC = 'app ntc'
    FILING_APP_ORDR = 'app ordr'
    FILING_APP_WD = 'app wd'
    FILING_ARS = 'ARS'
    FILING_ARS_AMEND = 'ARS/A'
    FILING_ATS = 'ats'
    FILING_ATSN = 'ATS-N'
    FILING_ATSNC = 'ATS-N-C'
    FILING_ATSNCA = 'ATS-N/CA'
    FILING_ATSNMA = 'ATS-N/MA'
    FILING_ATSNOFA = 'ATS-N/OFA'
    FILING_ATSNUA = 'ATS-N/UA'
    FILING_ATSNW = 'ATS-N-W'
    FILING_ATSN_A_ORDR_INEFF = 'ats-n/a ordr ineff'
    FILING_ATSN_CA = 'ats-n/ca'
    FILING_ATSN_MA = 'ats-n/ma'
    FILING_ATSN_MA_CP = 'ats-n/ma cp'
    FILING_ATSN_OFA = 'ats-n/ofa'
    FILING_ATSN_ORDR_INEFF = 'ats-n ordr ineff'
    FILING_ATSN_ORDR_LTD_PN = 'ats-n ordr ltd opn'
    FILING_ATSN_ORDR_REVK = 'ats-n ordr revk'
    FILING_ATSN_ORDR_SUSP = 'ats-n ordr susp'
    FILING_ATSN_UA = 'ats-n/ua'
    FILING_ATSR = 'ats-r'
    FILING_AW = 'AW'
    FILING_AWWD = 'AW WD'
    FILING_AW_WD = 'aw wd'
    FILING_BD = 'bd'
    FILING_BDCO = 'bdco'
    FILING_BDN = 'bd-n'
    FILING_BDW = 'bdw'
    FILING_BULK = 'BULK'
    FILING_BW2 = 'bw-2'
    FILING_BW3 = 'bw-3'
    FILING_C = 'C'
    FILING_CA1 = 'ca-1'
    FILING_CAR = 'C-AR'
    FILING_CARAW = 'C-AR/A-W'
    FILING_CARW = 'C-AR-W'
    FILING_CAR_AMEND = 'C-AR/A'
    FILING_CAW = 'C/A-W'
    FILING_CB = 'CB'
    FILING_CB_AMEND = 'CB/A'
    FILING_CERT = 'CERT'
    FILING_CERTAMX = 'certamx'
    FILING_CERTARCA = 'certarca'
    FILING_CERTBSE = 'certbse'
    FILING_CERTCBO = 'certcbo'
    FILING_CERTCIN = 'certcin'
    FILING_CERTCSE = 'certcse'
    FILING_CERTISE = 'certise'
    FILING_CERTNAS = 'certnas'
    FILING_CERTNYS = 'certnys'
    FILING_CERTPAC = 'certpac'
    FILING_CERTPBS = 'certpbs'
    FILING_CFPORTAL = 'CFPORTAL'
    FILING_CFPORTALW = 'CFPORTAL-W'
    FILING_CFPORTAL_AMEND = 'CFPORTAL/A'
    FILING_CORRESP = 'CORRESP'
    FILING_CTR = 'C-TR'
    FILING_CTRW = 'C-TR-W'
    FILING_CT_ORDER = 'ct order'
    FILING_CU = 'C-U'
    FILING_CUSTODY = 'custody'
    FILING_CUW = 'C-U-W'
    FILING_CW = 'C-W'
    FILING_C_AMEND = 'C/A'
    FILING_C_ARW = 'c-ar-w'
    FILING_C_AR_AW = 'c-ar/a-w'
    FILING_C_AW = 'c/a-w'
    FILING_C_TRW = 'c-tr-w'
    FILING_C_UW = 'c-u-w'
    FILING_D = 'D'
    FILING_DEF13E3 = 'def13e3'
    FILING_DEF14A = 'DEF 14A'
    FILING_DEF14C = 'DEF 14C'
    FILING_DEFA14A = 'DEFA14A'
    FILING_DEFA14C = 'DEFA14C'
    FILING_DEFC14A = 'DEFC14A'
    FILING_DEFC14C = 'DEFC14C'
    FILING_DEFM14A = 'DEFM14A'
    FILING_DEFM14C = 'DEFM14C'
    FILING_DEFN14A = 'DEFN14A'
    FILING_DEFOC = 'def-oc'
    FILING_DEFR14A = 'DEFR14A'
    FILING_DEFR14C = 'DEFR14C'
    FILING_DEFS14A = 'defs14a'
    FILING_DEFS14C = 'defs14c'
    FILING_DEF_14A = 'def 14a'
    FILING_DEF_14C = 'def 14c'
    FILING_DELAM = 'DEL AM'
    FILING_DEL_AM = 'del am'
    FILING_DFAN14A = 'DFAN14A'
    FILING_DFRN14A = 'DFRN14A'
    FILING_DOS = 'DOS'
    FILING_DOSLTR = 'DOSLTR'
    FILING_DOS_AMEND = 'DOS/A'
    FILING_DRS = 'DRS'
    FILING_DRSLTR = 'DRSLTR'
    FILING_DRS_AMEND = 'DRS/A'
    FILING_DSTRBRPT = 'DSTRBRPT'
    FILING_DSTRBRPT_AMEND = 'DSTRBRPT/A'
    FILING_D_AMEND = 'D/A'
    FILING_EBRD = 'ebrd'
    FILING_EFFECT = 'effect'
    FILING_F1 = 'F-1'
    FILING_F10 = 'F-10'
    FILING_F10EF = 'F-10EF'
    FILING_F10MEF = 'f-10mef'
    FILING_F10POS = 'F-10POS'
    FILING_F10_AMEND = 'F-10/A'
    FILING_F1MEF = 'F-1MEF'
    FILING_F1_AMEND = 'F-1/A'
    FILING_F2 = 'f-2'
    FILING_F2D = 'f-2d'
    FILING_F2DPOS = 'f-2dpos'
    FILING_F2MEF = 'f-2mef'
    FILING_F3 = 'F-3'
    FILING_F3ASR = 'F-3ASR'
    FILING_F3D = 'F-3D'
    FILING_F3DPOS = 'F-3DPOS'
    FILING_F3MEF = 'F-3MEF'
    FILING_F3_AMEND = 'F-3/A'
    FILING_F4 = 'F-4'
    FILING_F4EF = 'F-4EF'
    FILING_F4MEF = 'F-4MEF'
    FILING_F4POS = 'F-4 POS'
    FILING_F4_AMEND = 'F-4/A'
    FILING_F4_POS = 'f-4 pos'
    FILING_F6 = 'F-6'
    FILING_F6EF = 'F-6EF'
    FILING_F6POS = 'F-6 POS'
    FILING_F6_AMEND = 'F-6/A'
    FILING_F6_POS = 'f-6 pos'
    FILING_F7 = 'F-7'
    FILING_F7POS = 'F-7 POS'
    FILING_F7_AMEND = 'F-7/A'
    FILING_F7_POS = 'f-7 pos'
    FILING_F8 = 'F-8'
    FILING_F80 = 'F-80'
    FILING_F80POS = 'F-80POS'
    FILING_F80_AMEND = 'F-80/A'
    FILING_F8POS = 'F-8 POS'
    FILING_F8_AMEND = 'F-8/A'
    FILING_F8_POS = 'f-8 pos'
    FILING_F9 = 'f-9'
    FILING_F9EF = 'f-9ef'
    FILING_F9MEF = 'f-9mef'
    FILING_F9_POS = 'f-9 pos'
    FILING_FN = 'F-N'
    FILING_FN_AMEND = 'F-N/A'
    FILING_FOCUSN = 'focusn'
    FILING_FWP = 'FWP'
    FILING_FX = 'F-X'
    FILING_FX_AMEND = 'F-X/A'
    FILING_G405 = 'g-405'
    FILING_GFIN = 'g-fin'
    FILING_GFINW = 'g-finw'
    FILING_HISTORY = 'history'
    FILING_IADB = 'iadb'
    FILING_IBRD = 'ibrd'
    FILING_ID = 'id'
    FILING_IDNEWCIK = 'id-newcik'
    FILING_IFC = 'ifc'
    FILING_IRANNOTICE = 'IRANNOTICE'
    FILING_MA = 'MA'
    FILING_MAA = 'MA-A'
    FILING_MAI = 'MA-I'
    FILING_MAI_AMEND = 'MA-I/A'
    FILING_MANR = 'ma-nr'
    FILING_MAW = 'MA-W'
    FILING_MA_AMEND = 'MA/A'
    FILING_MODULE = 'MODULE'
    FILING_MSD = 'msd'
    FILING_MSDCO = 'msdco'
    FILING_MSDW = 'msdw'
    FILING_N1 = 'n-1'
    FILING_N14 = 'N-14'
    FILING_N148C = 'N-14 8C'
    FILING_N148C_AMEND = 'N-14 8C/A'
    FILING_N14AE = 'n-14ae'
    FILING_N14AE24 = 'n14ae24'
    FILING_N14EL24 = 'n14el24'
    FILING_N14MEF = 'N-14MEF'
    FILING_N14_8C = 'n-14 8c'
    FILING_N14_AMEND = 'N-14/A'
    FILING_N17D1 = 'n-17d-1'
    FILING_N17F1 = 'n-17f-1'
    FILING_N17F2 = 'n-17f-2'
    FILING_N18F1 = 'N-18F1'
    FILING_N18F1_AMEND = 'N-18F1/A'
    FILING_N1A = 'N-1A'
    FILING_N1A_AMEND = 'N-1A/A'
    FILING_N1A_EL = 'n-1a el'
    FILING_N2 = 'N-2'
    FILING_N23C1 = 'n-23c-1'
    FILING_N23C2 = 'N-23C-2'
    FILING_N23C2_AMEND = 'N-23C-2/A'
    FILING_N23C3 = 'n-23c-3'
    FILING_N23C3A = 'N-23C3A'
    FILING_N23C3A_AMEND = 'N23C3A/A'
    FILING_N23C3B = 'N-23C3B'
    FILING_N23C3B_AMEND = 'N23C3B/A'
    FILING_N23C3C = 'N-23C3C'
    FILING_N23C3C_AMEND = 'N23C3C/A'
    FILING_N27D1 = 'N-27D-1'
    FILING_N27D1_AMEND = 'N-27D-1/A'
    FILING_N2MEF = 'N-2MEF'
    FILING_N2_AMEND = 'N-2/A'
    FILING_N3 = 'N-3'
    FILING_N30B2 = 'N-30B-2'
    FILING_N30D = 'N-30D'
    FILING_N30D_AMEND = 'N-30D/A'
    FILING_N3_AMEND = 'N-3/A'
    FILING_N3_EL = 'n-3 el'
    FILING_N4 = 'N-4'
    FILING_N4_AMEND = 'N-4/A'
    FILING_N4_EL = 'n-4 el'
    FILING_N5 = 'N-5'
    FILING_N54A = 'N-54A'
    FILING_N54A_AMEND = 'N-54A/A'
    FILING_N54C = 'N-54C'
    FILING_N54C_AMEND = 'N-54C/A'
    FILING_N5_AMEND = 'N-5/A'
    FILING_N6 = 'N-6'
    FILING_N6C9 = 'n-6c9'
    FILING_N6EI1 = 'n-6ei-1'
    FILING_N6F = 'N-6F'
    FILING_N6F_AMEND = 'N-6F/A'
    FILING_N6_AMEND = 'N-6/A'
    FILING_N8A = 'N-8A'
    FILING_N8A_AMEND = 'N-8A/A'
    FILING_N8B2 = 'N-8B-2'
    FILING_N8B2_AMEND = 'N-8B-2/A'
    FILING_N8B3 = 'N-8B-3'
    FILING_N8B3_AMEND = 'N-8B-3/A'
    FILING_N8B4 = 'N-8B-4'
    FILING_N8B4_AMEND = 'N-8B-4/A'
    FILING_N8F = 'N-8F'
    FILING_N8F_AMEND = 'N-8F/A'
    FILING_N8F_NTC = 'n-8f ntc'
    FILING_N8F_ORDR = 'n-8f ordr'
    FILING_NCEN = 'N-CEN'
    FILING_NCEN_AMEND = 'N-CEN/A'
    FILING_NCR = 'N-CR'
    FILING_NCR_AMEND = 'N-CR/A'
    FILING_NCSR = 'N-CSR'
    FILING_NCSRS = 'N-CSRS'
    FILING_NCSRS_AMEND = 'N-CSRS/A'
    FILING_NCSR_AMEND = 'N-CSR/A'
    FILING_NLIQUID = 'N-LIQUID'
    FILING_NLIQUID_AMEND = 'N-LIQUID/A'
    FILING_NMFP = 'n-mfp'
    FILING_NMFP2 = 'N-MFP2'
    FILING_NMFP2NMFP2_AMEND = 'N-MFP2 N-MFP2/A'
    FILING_NMFP2_AMEND = 'N-MFP2/A'
    FILING_NOINFOBLANKCELL = 'No info blank cell'
    FILING_NO_ACT = 'no act'
    FILING_NPORT = 'nport'
    FILING_NPORTEX = 'NPORT-EX'
    FILING_NPORTNP = 'NPORT-NP'
    FILING_NPORTNP_AMEND = 'NPORT-NP/A'
    FILING_NPORTP = 'NPORT-P'
    FILING_NPORTP_AMEND = 'NPORT-P/A'
    FILING_NPX = 'N-PX'
    FILING_NPXCR = 'N-PX-CR'
    FILING_NPXCR_AMEND = 'N-PX-CR/A'
    FILING_NPXFM = 'N-PX-FM'
    FILING_NPXFM_AMEND = 'N-PX-FM/A'
    FILING_NPXNT = 'N-PX-NT'
    FILING_NPXNT_AMEND = 'N-PX-NT/A'
    FILING_NPXVR = 'N-PX-VR'
    FILING_NPXVR_AMEND = 'N-PX-VR/A'
    FILING_NPX_AMEND = 'N-PX/A'
    FILING_NQ = 'N-Q'
    FILING_NQ_AMEND = 'N-Q/A'
    FILING_NRSRO = 'nrsro'
    FILING_NRSROCE = 'NRSRO-CE'
    FILING_NRSROCE_AMEND = 'NRSRO- CE/A'
    FILING_NRSROFR = 'NRSRO-FR'
    FILING_NRSROFR_AMEND = 'NRSRO-FR/A'
    FILING_NRSROUPD = 'NRSRO-UPD'
    FILING_NRSROWCLS = 'NRSRO-WCLS'
    FILING_NRSROWREG = 'NRSRO-WREG'
    FILING_NSARA = 'NSAR-A'
    FILING_NSARAT = 'nsar-at'
    FILING_NSARB = 'NSAR-B'
    FILING_NSARBT = 'nsar-bt'
    FILING_NSARB_AMEND = 'NSAR-B/A'
    FILING_NSARU = 'nsar-u'
    FILING_NT10D = 'NT 10-D'
    FILING_NT10D_AMEND = 'NT 10-D/A'
    FILING_NT10K = 'NT 10-K'
    FILING_NT10K_AMEND = 'NT 10-K/A'
    FILING_NT10Q = 'NT 10-Q'
    FILING_NT10Q_AMEND = 'NT 10-Q/A'
    FILING_NT11K = 'NT 11-K'
    FILING_NT11K_AMEND = 'NT 11-K/A'
    FILING_NT15D2 = 'NT 15D2'
    FILING_NT15D2_AMEND = 'NT 15D2/A'
    FILING_NT20F = 'NT 20-F'
    FILING_NT20F_AMEND = 'NT 20-F/A'
    FILING_NTFNCEN = 'ntfncen'
    FILING_NTFNCSR = 'ntfncsr'
    FILING_NTFNSAR = 'ntfnsar'
    FILING_NTN15D2 = 'ntn15d2'
    FILING_NTNCEN = 'NT-NCEN'
    FILING_NTNCEN_AMEND = 'NT-NCEN/A'
    FILING_NTNCSR = 'NT-NCSR'
    FILING_NTNCSR_AMEND = 'NTNCSR/A'
    FILING_NTNSAR = 'nt-nsar'
    FILING_NTN_10D = 'ntn 10-d'
    FILING_NTN_10K = 'ntn 10k'
    FILING_NTN_10Q = 'ntn 10q'
    FILING_NTN_11K = 'ntn 11k'
    FILING_NTN_20F = 'ntn 20f'
    FILING_NT_10D = 'nt 10-d'
    FILING_NT_10K = 'nt 10-k'
    FILING_NT_10Q = 'nt 10-q'
    FILING_NT_11K = 'nt 11-k'
    FILING_NT_15D2 = 'nt 15d2'
    FILING_NT_20F = 'nt 20-f'
    FILING_NT_NCEN = 'nt n-cen'
    FILING_NT_NMFP = 'nt n-mfp'
    FILING_NT_NPORTEX = 'nt nport-ex'
    FILING_NT_NPORTN = 'nt nport-n'
    FILING_NT_NPORTP = 'nt nport-p'
    FILING_NVP = 'N-VP'
    FILING_NVPFS = 'N-VPFS'
    FILING_NVPFS_AMEND = 'N-VPFS/A'
    FILING_NVP_AMEND = 'N-VP/A'
    FILING_OC = 'oc'
    FILING_OIP_NTC = 'oip ntc'
    FILING_OIP_ORDR = 'oip ordr'
    FILING_PF = 'pf'
    FILING_PILOT = 'pilot'
    FILING_POS462B = 'POS462B'
    FILING_POS462C = 'POS462C'
    FILING_POS8C = 'POS 8C'
    FILING_POSAM = 'POS AM'
    FILING_POSAMI = 'POS AMI'
    FILING_POSASR = 'POSASR'
    FILING_POSEX = 'POS EX'
    FILING_POS_8C = 'pos 8c'
    FILING_POS_AM = 'pos am'
    FILING_POS_AMC = 'pos amc'
    FILING_POS_AMI = 'pos ami'
    FILING_POS_EX = 'pos ex'
    FILING_PRE13E3 = 'pre13e3'
    FILING_PRE14A = 'PRE 14A'
    FILING_PRE14C = 'PRE 14C'
    FILING_PREA14A = 'prea14a'
    FILING_PREA14C = 'prea14c'
    FILING_PREC14A = 'PREC14A'
    FILING_PREC14C = 'PREC14C'
    FILING_PREM14A = 'PREM14A'
    FILING_PREM14C = 'PREM14C'
    FILING_PREN14A = 'PREN14A'
    FILING_PRER14A = 'PRER14A'
    FILING_PRER14C = 'PRER14C'
    FILING_PRES14A = 'pres14a'
    FILING_PRES14C = 'pres14c'
    FILING_PRE_14A = 'pre 14a'
    FILING_PRE_14C = 'pre 14c'
    FILING_PRRN14A = 'PRRN14A'
    FILING_PWRATT = 'pwr-att'
    FILING_PX14A6G = 'PX14A6G'
    FILING_PX14A6N = 'PX14A6N'
    FILING_QRTLYRPT = 'QRTLYRPT'
    FILING_QRTLYRPT_AMEND = 'QRTLYRPT/A'
    FILING_QUALIF = 'qualif'
    FILING_R31 = 'r31'
    FILING_REGDEX = 'regdex'
    FILING_REGNR = 'reg-nr'
    FILING_REVOKED = 'revoked'
    FILING_RW = 'RW'
    FILING_RWWD = 'RW WD'
    FILING_RW_WD = 'rw wd'
    FILING_S1 = 'S-1'
    FILING_S11 = 'S-11'
    FILING_S11MEF = 'S-11MEF'
    FILING_S11_AMEND = 'S-11/A'
    FILING_S1MEF = 'S-1MEF'
    FILING_S1_AMEND = 'S-1/A'
    FILING_S2 = 's-2'
    FILING_S20 = 'S-20'
    FILING_S20_AMEND = 'S-20/A'
    FILING_S2MEF = 's-2mef'
    FILING_S3 = 'S-3'
    FILING_S3ASR = 'S-3ASR'
    FILING_S3D = 'S-3D'
    FILING_S3DPOS = 'S-3DPOS'
    FILING_S3MEF = 'S-3MEF'
    FILING_S3_AMEND = 'S-3/A'
    FILING_S4 = 'S-4'
    FILING_S4EF = 'S-4EF'
    FILING_S4MEF = 'S-4MEF'
    FILING_S4POS = 'S-4 POS'
    FILING_S4_AMEND = 'S-4/A'
    FILING_S4_POS = 's-4 pos'
    FILING_S6 = 'S-6'
    FILING_S6EL24 = 's-6el24'
    FILING_S6_AMEND = 'S-6/A'
    FILING_S8 = 'S-8'
    FILING_S8POS = 'S-8 POS'
    FILING_S8_POS = 's-8 pos'
    FILING_SB = 'S-B'
    FILING_SB1 = 'sb-1'
    FILING_SB1MEF = 'sb-1mef'
    FILING_SB2 = 'sb-2'
    FILING_SB2MEF = 'sb-2mef'
    FILING_SBMEF = 'S-BMEF'
    FILING_SBSE = 'SBSE'
    FILING_SBSEA = 'SBSE-A'
    FILING_SBSEA_AMEND = 'SBSE-A/A'
    FILING_SBSEBD = 'SBSE-BD'
    FILING_SBSEBD_AMEND = 'SBSE-BD/A'
    FILING_SBSEC = 'SBSE-C'
    FILING_SBSEW = 'SBSE-W'
    FILING_SBSE_AMEND = 'SBSE/A'
    FILING_SB_AMEND = 'S-B/A'
    FILING_SC13D = 'SC 13D'
    FILING_SC13DA = 'sc 13d/a'
    FILING_SC13D_AMEND = 'SC 13D/A'
    FILING_SC13E1 = 'SC 13E1'
    FILING_SC13E1_AMEND = 'SC 13E1/A'
    FILING_SC13E3 = 'SC 13E3'
    FILING_SC13E3_AMEND = 'SC 13E3/A'
    FILING_SC13E4F = 'SC13E4F'
    FILING_SC13E4F_AMEND = 'SC13E4F/A'
    FILING_SC13G = 'SC 13G'
    FILING_SC13GA = 'sc 13g/a'
    FILING_SC13G_AMEND = 'SC 13G/A'
    FILING_SC14D1F = 'SC14D1F'
    FILING_SC14D1F_AMEND = 'SC14D1F/A'
    FILING_SC14D9 = 'SC 14D9'
    FILING_SC14D9C = 'SC14D9C'
    FILING_SC14D9F = 'SC14D9F'
    FILING_SC14D9F_AMEND = 'SC14D9F/A'
    FILING_SC14D9_AMEND = 'SC 14D9/A'
    FILING_SC14F1 = 'SC 14F1'
    FILING_SC14F1_AMEND = 'SC 14F1/A'
    FILING_SC14N = 'SC 14N'
    FILING_SC14NS = 'SC 14N-S'
    FILING_SC14NS_AMEND = 'SC 14N-S/A'
    FILING_SC14N_AMEND = 'SC 14N/A'
    FILING_SCI = 'sci'
    FILING_SCTOC = 'SC TO-C'
    FILING_SCTOI = 'SC TO-I'
    FILING_SCTOI_AMEND = 'SC TO-I/A'
    FILING_SCTOT = 'SC TO-T'
    FILING_SCTOT_AMEND = 'SC TO-T/A'
    FILING_SC_13D = 'sc 13d'
    FILING_SC_13E1 = 'sc 13e1'
    FILING_SC_13E3 = 'sc 13e3'
    FILING_SC_13E4 = 'sc 13e4'
    FILING_SC_13G = 'sc 13g'
    FILING_SC_14D1 = 'sc 14d1'
    FILING_SC_14D9 = 'sc 14d9'
    FILING_SC_14F1 = 'sc 14f1'
    FILING_SC_14N = 'sc 14n'
    FILING_SC_14NS = 'sc 14n-s'
    FILING_SC_TOC = 'sc to-c'
    FILING_SC_TOI = 'sc to-i'
    FILING_SC_TOT = 'sc to-t'
    FILING_SD = 'SD'
    FILING_SDR = 'SDR'
    FILING_SDRA = 'SDR-A'
    FILING_SDRCCO = 'SDR-CCO'
    FILING_SDRCCO_AMEND = 'SDR-CCO/A'
    FILING_SDRW = 'SDR-W'
    FILING_SDR_AMEND = 'SDR/A'
    FILING_SD_AMEND = 'SD/A'
    FILING_SE = 'se'
    FILING_SEC_ACTION = 'sec action'
    FILING_SEC_STAFF_ACTION = 'sec staff action'
    FILING_SEC_STAFF_LETTER = 'sec staff letter'
    FILING_SF1 = 'SF-1'
    FILING_SF1MEF = 'SF-1MEF'
    FILING_SF1_AMEND = 'SF-1/A'
    FILING_SF3 = 'SF-3'
    FILING_SF3MEF = 'SF-3MEF'
    FILING_SF3_AMEND = 'SF-3/A'
    FILING_SHER = 'SH-ER'
    FILING_SHER_AMEND = 'SH-ER/A'
    FILING_SHNT = 'SH-NT'
    FILING_SHNT_AMEND = 'SH-NT/A'
    FILING_SIP = 'sip'
    FILING_SL = 'sl'
    FILING_SP15D2 = 'SP 15D2'
    FILING_SP15D2_AMEND = 'SP 15D2/A'
    FILING_SP_15D2 = 'sp 15d2'
    FILING_STOP_ORDER = 'stop order'
    FILING_SUPPL = 'SUPPL'
    FILING_T1 = 'T-1'
    FILING_T2 = 'T-2'
    FILING_T3 = 'T-3'
    FILING_T3_AMEND = 'T-3/A'
    FILING_T4 = 't-4'
    FILING_T6 = 'T-6'
    FILING_T6_AMEND = 'T-6/A'
    FILING_TA1 = 'TA-1'
    FILING_TA1_AMEND = 'TA-1/A'
    FILING_TA2 = 'TA-2'
    FILING_TA2_AMEND = 'TA-2/A'
    FILING_TACO = 'taco'
    FILING_TAW = 'TA-W'
    FILING_TCR = 'tcr'
    FILING_TH = 'th'
    FILING_TTW = 'ttw'
    FILING_U1 = 'u-1'
    FILING_U12IA = 'u-12-ia'
    FILING_U12IB = 'u-12-ib'
    FILING_U131 = 'u-13-1'
    FILING_U1360 = 'u-13-60'
    FILING_U13E1 = 'u-13e-1'
    FILING_U33S = 'u-33-s'
    FILING_U3A2 = 'u-3a-2'
    FILING_U3A31 = 'u-3a3-1'
    FILING_U57 = 'u-57'
    FILING_U5A = 'u5a'
    FILING_U5B = 'u5b'
    FILING_U5S = 'u5s'
    FILING_U6B2 = 'u-6b-2'
    FILING_U7D = 'u-7d'
    FILING_U9C3 = 'u-9c-3'
    FILING_UNDER = 'UNDER'
    FILING_UNDER_AMEND = 'UNDER/A'
    FILING_UPLOAD = 'upload'
    FILING_UR1 = 'u-r-1'
    FILING_WBAPP = 'wb-app'
    FILING_WDLREQ = 'wdl-req'
    FILING_X17A5 = 'X-17A-5'
    FILING_X17A5_AMEND = 'X-17A-5/A'
